<?php

class LiamW_LotsaBots_Model_Robots extends XenForo_Model
{
	protected $_robots;

	public function getAllRobots()
	{
		if (!$this->_robots)
		{
			$this->_robots = $this->fetchAllKeyed("SELECT * FROM liam_robots", 'robot_id');
		}

		return $this->_robots;
	}

	public function getRobotById($id)
	{
		return $this->_getDb()->fetchRow("SELECT * FROM liam_robots WHERE robot_id=?", array(
			$id
		));
	}

	public function getRobotsByIds(array $ids)
	{
		return $this->fetchAllKeyed("SELECT * FROM liam_robots WHERE robot_id IN ( " . $this->_getDb()
				->quote($ids) . ")", 'robot_id');
	}

	public function rebuildRobotsCache()
	{
		$allBots = $this->getAllRobots();

		$robotsArray = array();
		$robotsInfo = array();

		foreach ($allBots as $bot)
		{
			$bot['user_agent'] = strtolower($bot['user_agent']);
			$robotsArray[$bot['user_agent']] = $bot['user_agent'];
			$robotsInfo[$bot['user_agent']] = array(
				'title' => $bot['name'],
				'link' => $bot['link']
			);
		}

		$dataRegistryModel = $this->_getDataRegistryModel();

		$dataRegistryModel->set('lotsaBots_robotsArray', $robotsArray);
		$dataRegistryModel->set('lotsaBots_robotsInfo', $robotsInfo);
	}

	public function getRobotsForArray()
	{
		return $this->_getDataRegistryModel()->get('lotsaBots_robotsArray');
	}

	public function getRobotsInfoArray()
	{
		return $this->_getDataRegistryModel()->get('lotsaBots_robotsInfo');
	}

	public function userAgentUnique($userAgent)
	{
		$dbUserAgent = $this->_getDb()->fetchOne(
			"SELECT user_agent FROM liam_robots WHERE user_agent=?", $userAgent
		);

		return $dbUserAgent !== $userAgent;
	}

	public function exportRobots(array $robotIds)
	{
		if (empty($robotIds))
		{
			throw new XenForo_Exception('Robot ids cannot be empty!');
		}

		$document = new DOMDocument('1.0', 'utf-8');
		$document->formatOutput = true;

		$rootNode = $document->createElement('robots');
		$rootNode->setAttribute('version', LiamW_LotsaBots_Addon::VERSION_CODE);
		$document->appendChild($rootNode);

		$robots = $this->getRobotsByIds($robotIds);

		ksort($robots);

		foreach ($robots as $robot)
		{
			$robotNode = $document->createElement('robot');
			$robotNode->setAttribute('name', $robot['name']);
			$robotNode->setAttribute('link', $robot['link']);
			$robotNode->setAttribute('user_agent', $robot['user_agent']);

			$rootNode->appendChild($robotNode);
		}

		return $document;
	}

	public function importRobots(XenForo_Upload $file)
	{
		$fileName = $file->getTempFile();

		if (!file_exists($fileName) || !is_readable($fileName))
		{
			throw new XenForo_Exception(new XenForo_Phrase('liam_robots_invalid_file'), true);
		}

		try
		{
			$document = XenForo_Helper_DevelopmentXml::scanFile($fileName);
		} catch (Exception $e)
		{
			throw new XenForo_Exception(new XenForo_Phrase('liam_robots_file_not_valid_xml'), true);
		}

		return $this->_importXml($document);
	}

	protected function _importXml(SimpleXMLElement $xml)
	{
		if ($xml->getName() != 'robots')
		{
			throw new XenForo_Exception(new XenForo_Phrase('liam_robots_file_not_robots_file'), true);
		}

		if ((int)$xml['version'] > LiamW_LotsaBots_Addon::VERSION_CODE)
		{
			// There may be changes to the XML layout in newer versions, so don't allow import of XML from a newer version.
			throw new XenForo_Exception(new XenForo_Phrase('liam_robots_file_from_newer_version'), true);
		}

		$db = $this->_getDb();

		XenForo_Db::beginTransaction($db);

		$robotXml = XenForo_Helper_DevelopmentXml::fixPhpBug50670($xml);

		$invalidEntries = array();

		foreach ($robotXml as $robot)
		{
			$dw = XenForo_DataWriter::create('LiamW_LotsaBots_DataWriter_Robots', XenForo_DataWriter::ERROR_SILENT);
			$dw->bulkSet(array(
				'name' => (string)$robot['name'],
				'link' => (string)$robot['link'],
				'user_agent' => (string)$robot['user_agent']
			));
			$dw->save();

			if ($dw->hasErrors())
			{
				$invalidEntries[] = array(
					'robot' => $robot,
					'errors' => $dw->getErrors()
				);
			}
		}

		XenForo_Db::commit($db);

		return $invalidEntries;
	}

}